class BSTNode:
    def __init__(self,k,v):
        self.key = k
        self.value = v
        self.left = None
        self.right = None

class BSTree:
    def __init__(self):
        self.root = None

    def isEmpty(self):
        return self.root == None

    def get(self, k):
        n = self.root
        while n is not None:
            if n.key == k:
                return n.value
            elif n.key > k:
                n = n.left
            else:
                n = n.right
        return None

    def set(self, k, v):
        p = None
        n = self.root

        while n is not None:
            p = n
            if n.key == k:
                n.value = v
                return
            elif n.key > k:
                n = n.left
            else:
                n = n.right

        e = BSTNode(k,v)
        if p is None:
            self.root = e
        elif p.key > k:
            p.left = e
        else:
            p.right = e

    def minimumKey(self):
        if self.isEmpty():
            return None
        else:
            n = self.root
            while n.left is not None:
                n = n.left
            return n.key

    def size(self):

        def subtreeSize(n):
            if n is None:
                return 0
            else:
                return 1 + subtreeSize(n.left) + subtreeSize(n.right)

        return subtreeSize(self.root)

    def asList(self):

        def subtreeList(n):
            if n is None:
                return []
            else:
                return subtreeList(n.left) + [(n.key,n.value)] + subtreeList(n.right)

        return subtreeList(self.root)

    def asString(self):
        kvs = self.asList()
        if len(kvs) == 0:
            return "{}"
        else:
            s = "{"
            for kv in kvs[:-1]:
                s += repr(kv[0])+": "+repr(kv[1])+", "
            kv = kvs[-1]
            s += repr(kv[0])+": "+repr(kv[1])+"}"
            return s

    def __repr__(self):
        return self.asString()

    __str__ = __repr__

    def __bool__(self):
        return not self.isEmpty()

    def __contains__(self,k):
        return self.get(k) is not None

    def __len__(self):
        return self.size()

    def __getitem__(self,k):
        return self.get(k)

    def __setitem__(self,k,v):
        return self.set(k,v)
